using Passer.Humanoid.Tracking;
using UnityEngine;

namespace Passer.LinearAlgebra {

    public class Spherical16 {

        public float16 distance;
        public byte horizontal;
        public byte vertical;

        public Spherical16(float distance, float horizontal = 0, float vertical = 0) {
            horizontal = Angle.Normalize(horizontal);
            vertical = Angle.Normalize(vertical);
            this.distance = new float16(distance);
            this.horizontal = (byte)(horizontal * 128.0f / 180.0f);
            this.vertical = (byte)(vertical * 128.0f / 180.0f);
        }

        public static Spherical16 FromVector3(Vector3 v) {
            float distance = v.magnitude;
            if (distance == 0.0f) {
                return new Spherical16(distance);
            }
            else {
                float verticalAngle = (Mathf.PI / 2 - Mathf.Acos(v.y / distance)) * Mathf.Rad2Deg;
                float horizontalAngle = Mathf.Atan2(v.x, v.z) * Mathf.Rad2Deg;
                Debug.Log($"sph {horizontalAngle} {verticalAngle} {distance}");
                return new Spherical16(distance, horizontalAngle, verticalAngle);
            }
        }
    }

    public class Quat32 {
        public byte qx;
        public byte qy;
        public byte qz;
        public byte qw;

        public Quat32(Quaternion q) {
            int qx = (int)(q.x * 127 + 128);
            int qy = (int)(q.y * 127 + 128);
            int qz = (int)(q.z * 127 + 128);
            int qw = (int)(q.w * 255);
            if (q.w < 0) {
                qx = -qx;
                qy = -qy;
                qz = -qz;
                qw = -qw;
            }
            this.qx = (byte)(qx);
            this.qy = (byte)(qy);
            this.qz = (byte)(qz);
            this.qw = (byte)(qw);
        }
    }
}